<?php

namespace Drupal\content_reporting_charts\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller class for Content Reporting Charts.
 */
class ContentReportingChartsController extends ControllerBase {
  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * Factory method for creating an instance of the controller.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container from which the database service is retrieved.
   *
   * @return static
   *   An instance of the `ContentReportingChartsController` class.
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('database'),
      );
  }

  /**
   * Retrieves page views data from the 'content_reporting_reports' table.
   *
   * @return array
   *   An associative array containing:
   *     - 'titles': The list of page titles.
   *     - 'views': The corresponding view counts for each page.
   *     - 'gdpr_consent': The GDPR consent counts for each page.
   */
  public function getPageViewsData() {
    $connection = $this->database;
    $query = $connection->select('content_reporting_reports', 'r')
      ->fields('r', ['title', 'views', 'gdpr_consent'])
    // Sort by page title.
      ->orderBy('title', 'ASC')
      ->execute()
      ->fetchAllAssoc('title');

    $data = [
      'titles' => [],
      'views' => [],
      'gdpr_consent' => [],
    ];

    foreach ($query as $row) {
      $data['titles'][] = $row->title;
      $data['views'][] = (int) $row->views;
      $data['gdpr_consent'][] = $row->gdpr_consent ? (int) $row->gdpr_consent : 0;
    }

    return $data;
  }

  /**
   * Builds and returns the page views dashboard chart.
   *
   * @return array
   *   A renderable array containing the chart and its configurations.
   */
  public function dashboard() {
    $data = $this->getPageViewsData();

    // Render the chart element.
    $chart = [
      '#type' => 'chart',
      '#id' => 'content_reporting_page_views_chart',
      '#title' => $this->t('Views per Page'),
      '#chart_id' => 'content_reporting_page_views_chart',
    // Use a column chart to show views.
      '#chart_type' => 'column',
    ];

    // Set the series data.
    $chart['series'] = [
      '#type' => 'chart_data',
      '#title' => $this->t('Views'),
      '#data' => array_map(
              function ($title, $views) {
                  return ['name' => $title, 'y' => $views];
              },
              $data['titles'],
              $data['views']
      ),
    ];

    // Set the x-axis labels.
    $chart['x_axis'] = [
      '#type' => 'chart_xaxis',
      '#labels' => $data['titles'],
    ];

    // Set the y-axis label.
    $chart['y_axis'] = [
      '#type' => 'chart_yaxis',
      '#title' => $this->t('Counts'),
    ];

    return [
      '#theme' => 'content_reporting_charts_dashboard',
      '#chart' => $chart,
    ];
  }

}
