<?php

namespace Drupal\content_reporting\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Provides functionality for content reporting.
 *
 * Extends the base Controller class to manage report data and user permissions.
 */
class ContentReportingController extends ControllerBase {
  use StringTranslationTrait;
  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs a ContentReportingController object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   */
  public function __construct(AccountProxyInterface $current_user, Connection $database, LoggerInterface $logger, FormBuilderInterface $form_builder) {
    $this->currentUser = $current_user;
    $this->database = $database;
    $this->logger = $logger;
    $this->formBuilder = $form_builder;
  }

  /**
   * Creates an instance of the ContentReportingController.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   The controller instance.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('database'),
      $container->get('logger.factory')->get('content_reporting'),
      $container->get('form_builder')
    );
  }

  /**
   * Retrieves a report based on the provided filters.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object containing query parameters.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response containing the report data or an error message.
   */
  public function getReport(Request $request) {
    // Check user permissions to ensure they can view the report.
    if (!$this->currentUser->hasPermission('content reporting view')) {
      return new JsonResponse([
        'status' => 'error',
        'message' => 'Access denied',
        // Return 403 if the user does not have permission.
      ], 403);
    }

    // Get filters from the request query parameters.
    $filters = [
      'title' => $request->query->get('title'),
      'start_date' => $request->query->get('start_date'),
      'end_date' => $request->query->get('end_date'),
      'gdpr_consent' => $request->query->get('gdpr_consent'),
    ];

    // Define table headers with sorting attributes.
    $header = [
      ['data' => $this->t('Report ID'), 'field' => 'id', 'order' => 'id'],
      ['data' => $this->t('Page Title'), 'field' => 'title', 'order' => 'title'],
      ['data' => $this->t('Views'), 'field' => 'views', 'order' => 'views'],
      // Field not directly in the DB.
      ['data' => $this->t('Clicks'), 'field' => 'clicks'],
      // Calculated field.
      ['data' => $this->t('Time Spent'), 'field' => 'time_spent'],
      ['data' => $this->t('GDPR count'), 'field' => 'gdpr_consent', 'order' => 'gdpr_consent'],
    ];

    // Main query to get report data from the database.
    $query = $this->database->select('content_reporting_reports', 'r')
      ->fields('r', ['id', 'title', 'views', 'gdpr_consent', 'nid'])
      ->extend(PagerSelectExtender::class)
      ->limit(10);

    // Define sorting options based on the request query parameters.
    $sort = $request->query->get('order') ? $request->query->get('order') : 'id';
    $order = $request->query->get('sort') ? strtoupper($request->query->get('sort')) : 'ASC';
    if (!$order) {
      $order = 'ASC';
    }

    // Apply sorting to the query.
    if (!empty($sort) && in_array($sort, ['id', 'title', 'views', 'clicks', 'time_spent', 'gdpr_consent'])) {
      $query->orderBy($sort, $order);
    }
    else {
      // Default sorting by ID if no valid sorting option is provided.
      $query->orderBy('id', 'ASC');
    }

    // Apply filters to the query based on the request parameters.
    if (!empty($filters['title'])) {
      $query->condition('title', '%' . $filters['title'] . '%', 'LIKE');
    }
    if (!empty($filters['start_date'])) {
      $query->condition('report_date', strtotime($filters['start_date']), '>=');
    }
    if (!empty($filters['end_date'])) {
      $query->condition('report_date', strtotime($filters['end_date']), '<=');
    }

    // Filters based on selected GDPR consent ranges.
    if (!empty($filters['gdpr_consent'])) {
      // Add an expression to calculate the GDPR consent percentage.
      $query->addExpression('(r.gdpr_consent / r.views) * 100', 'gdpr_percentage');
      switch ($filters['gdpr_consent']) {
        case '0_cons':
          // No consent (0 consents).
          $query->condition('r.gdpr_consent', 0);
          break;

        case '0_25':
          // 0% to 25% consent.
          $query->having('gdpr_percentage >= :min', [':min' => 0])
            ->having('gdpr_percentage <= :max', [':max' => 25]);
          break;

        case '25_50':
          // 25% to 50% consent.
          $query->having('gdpr_percentage >= :min', [':min' => 25])
            ->having('gdpr_percentage <= :max', [':max' => 50]);
          break;

        case '50_70':
          // 50% to 70% consent.
          $query->having('gdpr_percentage >= :min', [':min' => 50])
            ->having('gdpr_percentage <= :max', [':max' => 70]);
          break;

        case '70_100':
          // 70% to 100% consent.
          $query->having('gdpr_percentage >= :min', [':min' => 70])
            ->having('gdpr_percentage <= :max', [':max' => 100]);
          break;
      }
    }

    // Execute the query and fetch results.
    $results = $query->execute()->fetchAll();

    // Prepare rows for the table based on query results.
    $rows = [];
    foreach ($results as $result) {
      // Query to get all "click" interactions for a specific node ID (nid).
      $clicks_query = $this->database->select('content_reporting_interactions', 'i')
        ->fields('i', ['nid', 'id', 'interaction_type', 'element'])
        ->condition('i.interaction_type', 'click')
        ->condition('i.nid', $result->nid);

      $clicks_result = $clicks_query->execute()->fetchAll();

      // Collect and count clicks per element.
      $clicks_data = [];
      foreach ($clicks_result as $click) {
        $element = strlen($click->element) > 5 ? substr($click->element, 0, 255) : $click->element;
        if (!isset($clicks_data[$element])) {
          $clicks_data[$element] = 0;
        }
        $clicks_data[$element]++;
      }

      // Generate HTML table for clicks.
      $clicks_table = '<table><tr><th>Element</th><th>Count</th></tr>';
      foreach ($clicks_data as $element => $count) {
        $clicks_table .= '<tr><td>' . $element . '</td><td>' . $count . '</td></tr>';
      }
      $clicks_table .= '</table>';

      // Get all "time_spent" interactions for a specific node ID (nid).
      $time_spent_query = $this->database->select('content_reporting_interactions', 'i')
        ->fields('i', ['nid', 'duration', 'interaction_type'])
        ->condition('i.interaction_type', 'time_spent')
        ->condition('i.nid', $result->nid);

      $time_spent_result = $time_spent_query->execute()->fetchAll();

      // Sum the "time_spent" in PHP.
      $time_spent = 0;
      foreach ($time_spent_result as $record) {
        $time_spent += $record->duration;
      }

      // Log the results for debugging purposes.
      $this->logger->info('Clicks for nid @nid: @clicks Time spent @time_spent', [
        '@nid' => $result->id,
        '@clicks' => Markup::create($clicks_table),
        '@time_spent' => $time_spent,
      ]);

      // Handle cases where there might be no data.
      $rows[] = [
        'data' => [
          $result->id,
          $result->title,
          $result->views,
          $clicks_result !== FALSE && $clicks_data ?
            [
              // Use Markup for HTML rendering.
              'data' => Markup::create($clicks_table),
              // Optional CSS class.
              'class' => ['clicks-column'],
            ]
            : 0,
          $time_spent !== FALSE ? $time_spent : 0,
          $result->gdpr_consent,
        ],
      ];
    }

    // Attach library for additional styles.
    $build['#attached']['library'][] = 'content_reporting/content_reporting.styles';

    // Add form elements for filters.
    $build['filters'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['filters-container']],
    ];

    // Optional: Add form and form API to manage filters.
    $build['filters']['filters_form'] = $this->formBuilder->getForm('Drupal\content_reporting\Form\ContentReportingFiltersForm');
    $build['filters']['export_link'] = [
      '#type' => 'link',
      '#title' => $this->t('Export to CSV'),
      '#url' => Url::fromRoute('content_reporting.export_to_csv', [], ['query' => $request->query->all()]),
      '#attributes' => [
        'class' => ['button'],
      ],
    ];

    // Build the table.
    $build['report'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No results found.'),
      '#attributes' => [
        'id' => 'content-reporting-table',
        'class' => ['content-reporting'],
      ],
    ];

    // Add pagination control.
    $build['pager'] = [
      '#type' => 'pager',
    ];

    return $build;
  }

  /**
   * Exports the report data to a CSV file.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object containing query parameters.
   *
   * @return \Symfony\Component\HttpFoundation\StreamedResponse
   *   A streamed response containing the CSV file.
   */
  public function exportToCsv(Request $request) {
    // Same logic as getReport() to get the filtered data.
    $filters = [
      'title' => $request->query->get('title'),
      'start_date' => $request->query->get('start_date'),
      'end_date' => $request->query->get('end_date'),
      'gdpr_consent' => $request->query->get('gdpr_consent'),
    ];

    $query = $this->database->select('content_reporting_reports', 'r')
      ->fields('r', ['id', 'title', 'views', 'gdpr_consent', 'nid']);

    // Apply filters to the query.
    if (!empty($filters['title'])) {
      $query->condition('title', '%' . $filters['title'] . '%', 'LIKE');
    }
    if (!empty($filters['start_date'])) {
      $query->condition('report_date', strtotime($filters['start_date']), '>=');
    }
    if (!empty($filters['end_date'])) {
      $query->condition('report_date', strtotime($filters['end_date']), '<=');
    }

    // Execute the query and fetch results.
    $results = $query->execute()->fetchAll();

    // Prepare CSV response.
    $response = new StreamedResponse(function () use ($results) {
      $handle = fopen('php://output', 'w+');
      fputcsv($handle, ['ID', 'Title', 'Views', 'GDPR Consent']);

      foreach ($results as $result) {
        fputcsv($handle, [$result->id, $result->title, $result->views, $result->gdpr_consent]);
      }

      fclose($handle);
    });

    // Set headers for the CSV download.
    $response->headers->set('Content-Type', 'text/csv');
    $response->headers->set('Content-Disposition', 'attachment; filename="content_report.csv"');

    return $response;
  }

}
