<?php

namespace Drupal\content_reporting\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Defines the form for filtering content reports.
 *
 * Extends Drupal's FormBase to leverage the form API functionality.
 */
class ContentReportingFiltersForm extends FormBase {

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a ContentReportingFiltersForm object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   */
  public function __construct(RequestStack $request_stack) {
    $this->requestStack = $request_stack;
  }

  /**
   * Creates an instance of the form.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   The form instance.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack')
    );
  }

  /**
   * Returns the unique ID of the form.
   *
   * @return string
   *   The unique form ID.
   */
  public function getFormId() {
    return 'content_reporting_filters_form';
  }

  /**
   * Builds the filter form for content reporting.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   An object containing the current state of the form.
   *
   * @return array
   *   The renderable form array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Access the request directly.
    $request = $this->requestStack->getCurrentRequest();

    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Page Title'),
      '#default_value' => $request->query->get('title'),
    ];

    $form['start_date'] = [
      '#type' => 'date',
      '#title' => $this->t('Start Date'),
      '#default_value' => $request->query->get('start_date'),
    ];

    $form['end_date'] = [
      '#type' => 'date',
      '#title' => $this->t('End Date'),
      '#default_value' => $request->query->get('end_date'),
    ];

    $form['gdpr_consent'] = [
      '#type' => 'select',
      '#title' => $this->t('GDPR Consent'),
      '#options' => [
        '' => $this->t('- Any -'),
        '0_cons' => $this->t('0% Consents'),
        '0_25' => $this->t('0% - 25% Consents'),
        '25_50' => $this->t('25% - 50% Consents'),
        '50_70' => $this->t('50% - 70% Consents'),
        '70_100' => $this->t('70% - 100% Consents'),
      ],
      '#default_value' => $request->query->get('gdpr_consent'),
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Filter'),
        '#button_type' => 'primary',
      ],
      'reset' => [
        '#type' => 'submit',
        '#value' => $this->t('Reset'),
        '#button_type' => 'secondary',
        '#submit' => ['::resetForm'],
      ],
    ];

    return $form;
  }

  /**
   * Handles the form submission.
   *
   * Processes the data submitted through the filter form.
   *
   * @param array &$form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $query = $form_state->getValues();
    $query['title'] = $form_state->getValue('title');
    $query['start_date'] = $form_state->getValue('start_date');
    $query['end_date'] = $form_state->getValue('end_date');
    $query['gdpr_consent'] = $form_state->getValue('gdpr_consent');

    $form_state->setRedirect('content_reporting.dashboard', $query);
  }

  /**
   * Custom submit handler for the reset button.
   */
  public function resetForm(array &$form, FormStateInterface $form_state) {
    // Redirect to the form with default values (empty query parameters)
    $form_state->setRedirect('content_reporting.dashboard');
  }

}
